/**
 * Textured Sphere 
 * by Mike 'Flux' Chang (cleaned up by Aaron Koblin). 
 * Based on code by Toxi. 
 * 
 * A 3D textured sphere with simple rotation control.
 * Note: Controls will be inverted when sphere is upside down. 
 * Use an "arc ball" to deal with this appropriately.
 *
 * Some tweaks by v3ga @ EESI in Poitiers, January 2010
 *
 * Controls : 
 * Q - move camera towards 
 * W - move camera backwards
 * Arrows : move camera into its near plane
 * CTRL + mouse drag : make the earth rotate
 */ 
 
 
 
import processing.opengl.*;
import damkjer.ocd.*;

PImage bg;
PImage texmap;
PFont font;

float rotationX = 25;
float rotationY = 0;
float velocityX = 0;
float velocityY = 0;
float globeRadius = 450;

ArrayList particlesSystemGeo;
ArrayList travellers;
CameraPilot cam;

float gx = 0;
float gy = 1;
float gz = 0;


void setup() 
{
  size(800, 600, OPENGL);  
  texmap = loadImage("world32k.jpg");    
  font = loadFont("HelveticaNeue-Bold-14.vlw");
  initializeSphere(sDetail);

  float dglobeRadius=(globeRadius+240)*0.33; // Not sure why (see Sphere code)

  particlesSystemGeo = new ArrayList();
  particlesSystemGeo.add( new ParticleSystemGeo("Poitiers", 30, 46.0, 0.06, dglobeRadius) );
  particlesSystemGeo.add( new ParticleSystemGeo("Dakar",    30, 14.72, -17.48, dglobeRadius) );
  particlesSystemGeo.add( new ParticleSystemGeo("NYC",      30, 40.5, -73.58, dglobeRadius) );
  particlesSystemGeo.add( new ParticleSystemGeo("Beyrouth", 30, 33, 35, dglobeRadius) );
  particlesSystemGeo.add( new ParticleSystemGeo("Rio",      30, -22, -43, dglobeRadius) );
  
  travellers = new ArrayList();
  ParticleSystemGeo A = (ParticleSystemGeo) particlesSystemGeo.get(0);
  for (int i=1;i<particlesSystemGeo.size()-1;i++)
  {
    ParticleSystemGeo B = (ParticleSystemGeo) particlesSystemGeo.get(i);
    travellers.add (new GeoTraveller(A.geoBasis, B.geoBasis, 1.3));
  }

  cam = new CameraPilot(this, PI/3, (float)width/(float)height, 0.1, 5000);
  cam.jump(0,0,600);

}

void draw() 
{    
  background(0);            
  renderGlobe(); 
}

void renderGlobe() 
{
  pushMatrix();
  cam.feed();
  pushMatrix();
  noFill();
  stroke(255,200);
  strokeWeight(2);
  smooth();
  popMatrix();
  lights();    
  pushMatrix();
  rotateX( radians(-rotationX) );  
  rotateY( radians(270 - rotationY) );
  fill(200);
  noStroke();
  textureMode(IMAGE);  
  texturedSphere(globeRadius, texmap);
  rotationX += velocityX;
  rotationY += velocityY;
  velocityX *= 0.95;
  velocityY *= 0.95;
  
  
  noLights();
  ParticleSystemGeo p;
  for (int i=0;i<particlesSystemGeo.size();i++)
  {
    p = (ParticleSystemGeo) particlesSystemGeo.get(i);
    p.update();
    p.draw();
  }
  
  for (int i=0;i<travellers.size();i++)
  {
    GeoTraveller t = (GeoTraveller) travellers.get(i);
    t.update();
    t.draw();
  }

  popMatrix();  
  popMatrix();
  
  // Implements mouse control (interaction will be inverse when sphere is  upside down)
  if(key == CODED && mousePressed){
    velocityX += (mouseY-pmouseY) * 0.01;
    velocityY -= (mouseX-pmouseX) * 0.01;
  }
}


